#ifndef EXTDEQUEUE_HPP
#define EXTDEQUEUE_HPP

#include <iostream>
#include <deque>
#include <map>

/*!
** \brief
** Classe utilitaire, drive de std::deque
*/
template<typename T>
class extdeque : public std::deque<T>
{
public:
	extdeque() {}
	virtual ~extdeque() {}

	/*!
	** \brief
	** Compte le nombre d'occurences d'une valeur dans la queue
	** @param Template val	: valeur pour laquelle compter toutes les occurences
	** @return Integer		: nombre d'occurences de val dans la queue
	*/
	size_t		count(const T& val) const
	{
		size_t	c = 0;

		for (size_t i = 0; i < this->size(); ++i) {
			if (this->at(i) == val)
				++c;
		}
		return c;
	}

	/*!
	** \brief
	** Renvoie la valeur qui totalise le plus grand nombre d'occurences
	** @return Template	:	valeur stocke le plus grand nombre de fois
	*/
	T						star_value() const
	{
		T					val = static_cast<T>(NULL);
		size_t				counter = 0;
		std::map<T, size_t>	tmp;

		for (size_t i = 0; i < this->size(); ++i) {
			const T& c = this->at(i);

			if (tmp.find(c) == tmp.end()) {
				tmp[c] = this->count(c);
				if (counter < tmp[c]) {
					counter = tmp[c];
					val = c;
				}
			}
		}
		return val;
	}
};

#endif // !EXTDEQUEUE_HPP