#ifndef EYETRACK_H
#define EYETRACK_H

#include <QMutex>
#include <QObject>
#include <QWidget>
#include <opencv2/opencv.hpp>
#include "def.h"
#include "eyes.h"
#include "eyes_actions_callback.h"

namespace eyetrack
{

/*!
** \brief class EyeTrack
** Classe de contrle du module eye-tracking
*/
class EyeTrack : public QObject
{
	Q_OBJECT

public:
	/*!
	** \brief Largeur du flux video (en px) 
	*/
	static const double			FRAME_WIDTH;
	/*!
	** \brief Hauteur du flux video (en px) 
	*/
	static const double			FRAME_HEIGHT;
	
	static const std::string	WINDOW_NAME;

	/*!
	** \brief Constructeur
	** @param class Callback		: contient tous les signaux  utiliser dans le module eye-tracking
	** @param String folderPath		: chemin vers le dossier de resources (ou seront stocks et utiliss toutes les resources ncessaires au fonctionnement du module)
	** @param QObject parent (NULL)	: l'objet parent
	*/
	EyeTrack(eyes_actions_callback& callback, const QString& folderPath, QObject* parent = 0);
	
	virtual ~EyeTrack();

public slots:
	/*!
	** \brief
	** Lance la boucle principale d'vnement
	** @return Boolean : true si aucune erreur materielle ou logicielle n'est dtecte
	*/
	bool					run_tracker();
	/*!
	** \brief
	** Arrete la boucle, libre le priphrique de capture
	*/
	void					stop_tracker();

private:
	cv::VideoCapture		mCapture;
	ObjectDetect*			mFace;
	Eyes*					mEyes;
	eyes_actions_callback&	mCallback;
	bool					mOpened;
	QMutex					mMutex;

	void					showAll(cv::Mat& frame);
	/*!
	** \brief
	** Dmarre l'utilisation classique du module
	*/
	void					track();
	/*!
	** \brief
	** Dmarre la phase de calibrage pour les yeux
	*/
	void					trainingProcess();
	/*!
	** Essaie de localiser les yeux dans l'image capture par la camra
	** @param Integer timeout : temps maximal en ms pour localiser les yeux, -1 (dfaut) pour aucune limite
	*/
	void					locateEyes(const qint64 timeout = -1);

signals:
	void					finished();
};

}

#endif // !EYETRACK_H