#ifndef GUI_H
#define GUI_H

//#include <QApplication>
//#include <QDesktopWidget>
#include <QLabel>
#include <QPixmap>
#include <QMap>
#include <QString>
#include "def.h"

using namespace eyetrack;

class MainController;

/*!
** \brief
** Helper class for drawing simple image to a transparent window anywhere on the screen
*/
template<Direction dir>
class LookAtWidget : public QWidget
{
public:

	LookAtWidget(const QString& folderPath, QWidget* parent = 0) : QWidget(parent)
	{
		setWindowFlags(Qt::FramelessWindowHint);
		setAttribute(Qt::WA_TranslucentBackground);
		setMaximumSize(100, 100);

		mActive.setParent(this);
		mInactive.setParent(this);
		
		load(folderPath);

		mIsActive = true;
		setActive(false);
	}

	virtual ~LookAtWidget() {}

	void	load(const QString& folderPath)
	{
		switch (dir)
		{
		case TOP:
			mActive.setPixmap(QPixmap(folderPath + "/resources/gui/top_selected.png"));
			mInactive.setPixmap(QPixmap(folderPath + "/resources/gui/top.png"));
			break ;
		case RIGHT:
			mActive.setPixmap(QPixmap(folderPath + "/resources/gui/right_selected.png"));
			mInactive.setPixmap(QPixmap(folderPath + "/resources/gui/right.png"));
			break ;
		case BOTTOM:
			mActive.setPixmap(QPixmap(folderPath + "/resources/gui/bottom_selected.png"));
			mInactive.setPixmap(QPixmap(folderPath + "/resources/gui/bottom.png"));
			break ;
		case LEFT:
			mActive.setPixmap(QPixmap(folderPath + "/resources/gui/left_selected.png"));
			mInactive.setPixmap(QPixmap(folderPath + "/resources/gui/left.png"));
			break ;
		}
	}

	void	setActive(bool active)
	{
		if (mIsActive != active)
		{
			mIsActive = active;
			if (active)
			{
				mInactive.setVisible(false);
				mActive.setVisible(true);
			}
			else
			{
				mActive.setVisible(false);
				mInactive.setVisible(true);
			}
		}
	}

	bool	isActive() const
	{
		return mIsActive;
	}

private:
	QLabel	mActive;
	QLabel	mInactive;
	bool	mIsActive;
};

/*!
** \brief
** Displays the component of the EyeTracking Gui
** Those components are the widgets showing the direction pointed by the user's eyes
** The functional is based on slots, one for each of the functionalities
*/
class	Gui : public QWidget
{
	Q_OBJECT

public:

	Gui(const QString& folderPath, QObject* parent);
	virtual ~Gui();

public slots:

	void					Update(const Direction dir);
	
	void					Slt_LookTop();
	void					Slt_LookRight();
	void					Slt_LookBottom();
	void					Slt_LookLeft();
	void					Slt_LookUnknown();
	void					Slt_DisplayHelp(bool status);
	void					Slt_DisplayNoDevice();
	void					Slt_DisplayEyesProblem(bool status);

private:

	LookAtWidget<TOP>*		mDirTop;
	LookAtWidget<RIGHT>*	mDirRight;
	LookAtWidget<BOTTOM>*	mDirBottom;
	LookAtWidget<LEFT>*		mDirLeft;
	QLabel*					mHelpLabel;
	QLabel*					mEyesProblemLabel;
	QLabel*					mNoDeviceLabel;
	Direction				mLookAt;
	MainController&			mMainController;
	bool					mIsShowingTips;

	void					showTips(bool);
};

#endif // !GUI_H
