/* IOT Copyright (C) 2010 CEG development team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Authors: CEG <ceg@ionlythink.com>, http://www.ionlythink.com
 */

#ifndef SETTINGSDIALOG_H
#define SETTINGSDIALOG_H

#include <iostream>
#include <list>
#include <QDialog>
#include <QVector>
#include <QStringListModel>

#include "Box.h"
#include "Profil.h"
#include "AddProfilWindow.h"
#include "ui_settingsDialog.h"

class MainController;
/*! \class SettingsDialog
   * \brief Classe de Configuration
   *
   *  Permet de parametrer l'application via une interface graphique.
   */
class SettingsDialog : public QDialog, private Ui_SettingsDialog
{
    Q_OBJECT
public:
	/*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SettingsDialog
     */
    SettingsDialog(MainController *_mainC, QWidget *parent);
	/*!
     *  \brief Ajoute les datas dans le QlistView des profils
     */
	void fillXMLProfil(QString filename, Profil *profil);

public slots:
	/*!
     *  \brief ajoute un profil au QListView
     */
	void addProfil();
	/*!
     *  \brief Set le profil Courant
     */
	void loadProfil();
	/*!
     *  \brief Supprime un profil du QListView
     */
	void removeProfil();
	/*!
     *  \brief Supprime un Profil en Fonction de son nom
     */
	void removeProfilByName(const QString& name);
	/*!
     *  \brief Edit un profil
     */
	void editProfil();
	void				addProgram();
	void				removeProgram();
    void on_confList_currentItemChanged(QListWidgetItem*, QListWidgetItem*);
    void on_buttonBox_accepted();
    void on_buttonBox_rejected();
    void on_colorBlurButton_clicked();
    void on_colorFocusButton_clicked();
    void on_colorOpacitySlider_valueChanged(int value);
    void on_colorTextButton_clicked();

private:
    typedef QVector<QString>	Languages;
	QStringList					_profilsName; /*!< Liste des noms des profils */
	QStringListModel			*_profilModel; /*!< Model du QListView */
	MainController				*_mainC; /*!< Pointeur sur le MainController */
	QAbstractItemModel			*_programsModel;
	bool						_programModified;
private:
    Languages _languages;
};

#endif // SETTINGSDIALOG_H
