#include <QApplication>
#include <QDesktopWidget>
#include <iostream>
#include <opencv2/opencv.hpp>
#include "gui.h"
#include "MainController.h"
#include "Utils.h"

using namespace	eyetrack;

Gui::Gui(const QString& folderPath, QObject* parent) : mMainController(dynamic_cast<MainController&>(*parent))
{
	int			width, height, win_size = 100;

	/* Rcupration de la rsolution de l'cran */
	width		= QApplication::desktop()->screenGeometry().width();
	height		= QApplication::desktop()->screenGeometry().height();
	mDirTop		= new LookAtWidget<TOP>(folderPath, this);
	mDirRight	= new LookAtWidget<RIGHT>(folderPath, this);
	mDirBottom	= new LookAtWidget<BOTTOM>(folderPath, this);
	mDirLeft	= new LookAtWidget<LEFT>(folderPath, this);
	mLookAt		= UNKNOWN;

	/* Dplacement des fenetres au bon endroit */
	mDirTop->move(width / 2 - win_size / 2, 10);
	mDirRight->move(width - win_size - 10, height / 2 - win_size / 2);
	mDirBottom->move(width / 2 - win_size / 2, height - win_size - 10);
	mDirLeft->move(10, height / 2 - win_size / 2);

	setMinimumSize(QSize(width, height));
	move(0, 0);
	setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
	setAttribute(Qt::WA_TranslucentBackground);
	setAttribute(Qt::WA_Disabled);
	show();

	mHelpLabel = new QLabel(this);
	mEyesProblemLabel = new QLabel(this);
	mNoDeviceLabel = new QLabel(this);

	mHelpLabel->setText(tr("Procdure de calibrage des yeux\nRegardez les fleches qui s'allument en bleu  tour de rle"));
	mEyesProblemLabel->setText(tr("Le logiciel ne peut dtecter correctement vos yeux. Veuillez vrifier votre position par rapport  l'cran (60cm environ)\nUn recalibrage va s'excuter."));
	mNoDeviceLabel->setText(tr("Aucune camra dtecte. Veuillez vrifier le bon fonctionnement de votre matriel."));

	mHelpLabel->setStyleSheet("QLabel { font-size: 36px; font-weight: bold; color: #F00; }");
	mEyesProblemLabel->setStyleSheet("QLabel { font-size: 36px; color: #F00; }");
	mNoDeviceLabel->setStyleSheet("QLabel { font-size: 36px; color: #F00; }");

	mHelpLabel->hide();
	mEyesProblemLabel->hide();
	mNoDeviceLabel->hide();

	mHelpLabel->setAlignment(Qt::AlignCenter);
	mEyesProblemLabel->setAlignment(Qt::AlignCenter);
	mNoDeviceLabel->setAlignment(Qt::AlignCenter);

	mHelpLabel->setGeometry(120, 0, width - 240, height);
	mEyesProblemLabel->setGeometry(120, 0, width - 240, height);
	mNoDeviceLabel->setGeometry(120, 0, width - 240, height);

	mEyesProblemLabel->setWordWrap(true);
	mNoDeviceLabel->setWordWrap(true);
	mHelpLabel->setWordWrap(true);

	mIsShowingTips = false;
}

Gui::~Gui()
{
	delete mDirTop;
	delete mDirRight;
	delete mDirBottom;
	delete mDirLeft;
	delete mHelpLabel;
	delete mEyesProblemLabel;
	delete mNoDeviceLabel;
}

void	Gui::Update(const Direction dir)
{
	if (dir != mLookAt || dir == UNKNOWN)
	{
		mDirTop->setActive(false);
		mDirRight->setActive(false);
		mDirBottom->setActive(false);
		mDirLeft->setActive(false);

		switch (dir)
		{
		case TOP:
			mDirTop->setActive(true);
			break ;
		case RIGHT:
			mDirRight->setActive(true);
			break ;
		case BOTTOM:
			mDirBottom->setActive(true);
			break ;
		case LEFT:
			mDirLeft->setActive(true);
			break;
		case UNKNOWN:
			break ;
		}
		mLookAt = dir;
	}
}

void	Gui::Slt_LookBottom()
{
	Update(BOTTOM);
}

void	Gui::Slt_LookLeft()
{
	Update(LEFT);
}

void	Gui::Slt_LookRight()
{
	Update(RIGHT);
}

void	Gui::Slt_LookTop()
{
	Update(TOP);
}

void	Gui::Slt_LookUnknown()
{
	Update(UNKNOWN);
}

void	Gui::showTips(bool val)
{
	if (val)
	{
		if (mIsShowingTips == false)
		{
			mMainController.getView().hide();
			mIsShowingTips = true;
		}
	}
	else
	{
		if (!mHelpLabel->isVisible() && !mNoDeviceLabel->isVisible() && !mEyesProblemLabel->isVisible())
			if (mIsShowingTips == true)
			{
				mMainController.getView().show();
				mIsShowingTips = false;
			}
	}
}

void	Gui::Slt_DisplayHelp(bool status)
{
	mHelpLabel->setVisible(status);
	showTips(status);
}

void	Gui::Slt_DisplayEyesProblem(bool status)
{
	mEyesProblemLabel->setVisible(status);
	showTips(status);
}

void	Gui::Slt_DisplayNoDevice()
{
	mNoDeviceLabel->show();
	showTips(true);
}
