/* IOT Copyright (C) 2010 CEG development team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Authors: CEG <ceg@ionlythink.com>, http://www.ionlythink.com
 */

/*********************************/
#include <QSystemTrayIcon>
#include <QMenu>
#include <QAction>
#include <QIcon>
#include <QMessageBox>
#include <QApplication>
#include <QSettings>
/*********************************/
#include "SettingsDialog.h"
/*********************************/
#include "Systray.h"
/*********************************/
#include "Logger.h"
/*********************************/

Systray::Systray(QWidget *parent) :
	QWidget(parent), _mainC(*this), _trayIcon(0), _trayIconMenu(0), _startAction(0), _settingAction(0), _aboutQtAction(0), _aboutCegAction(0), _quitAction(0)
{
    this->_trayIcon = new QSystemTrayIcon(QIcon(":/images/systray-transparent-32x32.png"), this);
    this->_trayIconMenu = new QMenu(this);
    this->_startAction = new QAction(tr("Start"), this);
    this->_settingAction = new QAction(tr("Setting"), this);
    this->_aboutQtAction = new QAction(tr("About Qt"), this);
    this->_aboutCegAction = new QAction(tr("About Ceg"), this);
    this->_quitAction = new QAction(tr("Quit"), this);
    this->_startEyeTrackingAction = new QAction(tr("Enable EyeTracking"), this);

    this->_trayIconMenu->addAction(this->_startEyeTrackingAction);
    this->_trayIconMenu->addAction(this->_startAction);
    this->_trayIconMenu->addAction(this->_settingAction);
    this->_trayIconMenu->addAction(this->_aboutCegAction);
    this->_trayIconMenu->addAction(this->_aboutQtAction);
    this->_trayIconMenu->addSeparator();
    this->_trayIconMenu->addAction(this->_quitAction);

    this->_trayIcon->setContextMenu(this->_trayIconMenu);
    qApp->connect(this->_quitAction, SIGNAL(triggered()), SLOT(quit()));
    this->connect(this->_startAction, SIGNAL(triggered()), SLOT(on__startAction_triggered()));
    this->connect(this->_aboutCegAction, SIGNAL(triggered()), SLOT(on__aboutCegAction_triggered()));
    this->connect(this->_aboutQtAction, SIGNAL(triggered()), SLOT(on__aboutQtAction_triggered()));
    this->connect(this->_settingAction, SIGNAL(triggered()), SLOT(on__settingAction_triggered()));
    this->connect(this->_startEyeTrackingAction, SIGNAL(triggered()), SLOT(on__startEyeTracking_triggered()));

    QObject::connect(this, SIGNAL(navigationStarted()), &(this->_mainC), SLOT(on_start_navigation()));
    QObject::connect(this, SIGNAL(navigationStoped()), &(this->_mainC), SLOT(on_stop_navigation()));
    QObject::connect(this, SIGNAL(eyeTrackingStarted()), &(this->_mainC), SLOT(on_start_eyetracking()));
    QObject::connect(this, SIGNAL(eyeTrackingStopped()), &(this->_mainC), SLOT(on_stop_eyetracking()));

    this->setWindowIcon(QIcon(":/images/systray-transparent-32x32.png"));

    this->_trayIcon->show();

#ifdef _WIN32
    // FIXME: we are getting a display bug issue when the message appears on the screen
    // on Linux. It appears on the left but it must be on the right.
    this->_trayIcon->showMessage(tr("Information Message"), tr("Click on Start to launch the Default Navigator"), QSystemTrayIcon::MessageIcon(), 7000);
#endif
}

Systray::~Systray()
{
    delete this->_quitAction;
    delete this->_aboutQtAction;
    delete this->_aboutCegAction;
    delete this->_settingAction;
    delete this->_startAction;
    delete this->_trayIconMenu;
    delete this->_trayIcon;
    delete this->_startEyeTrackingAction;
    Logger::destroy();
}

void Systray::on__startAction_triggered()
{
    QString content;
    if (this->_startAction->text() == tr("Start"))
    {
	//QMessageBox::information(this, tr("Commandes"), tr("Left | Right arrow = Horizontal Move\nUp | Down Arrow = Vertical move\nEnter = Validate\nAlt + F4 = Quit"));
	emit navigationStarted();
	content = tr("Stop");
    }
    else
    {
	emit navigationStoped();
	content = tr("Start");
    }
    this->_startAction->setText(content);
}

void		Systray::on__startEyeTracking_triggered()
{
  QString	content;

  if (this->_startEyeTrackingAction->text() == tr("Enable EyeTracking"))
    {
      content = tr("Disable EyeTracking");
      emit eyeTrackingStarted();
    }
  else
    {
      content = tr("Enable EyeTracking");
      emit eyeTrackingStopped();
    }
  this->_startEyeTrackingAction->setText(content);
}

void Systray::on__aboutQtAction_triggered()
{
    QMessageBox::aboutQt(this, tr("About Qt"));
}

void Systray::on__settingAction_triggered()
{
	SettingsDialog settings(&this->_mainC, this);
    settings.exec();
}

void Systray::on__aboutCegAction_triggered()
{
    QMessageBox::about(this, tr("About CEG"), tr("Allows users to control machine by thinking."));
}
