// XAction.cpp

#ifndef _WIN32
# include "XAction.h"

XAction::XAction()
{
  this->main_display = ::XOpenDisplay(NULL);

  if (!this->main_display)
    {
      std::cerr << "unable to open a display" << std::endl;
      std::exception e;
      throw e;
    }

  this->root_window = DefaultRootWindow(this->main_display);
  ::XSelectInput(this->main_display, this->root_window, \
		 ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|OwnerGrabButtonMask);
}

XAction::~XAction()
{
  ::XCloseDisplay(this->main_display);
}

void XAction::keyPress(int k)
{
  bool propagate = true;
  KeySym symb = k;
  XEvent event;

  memset(&event, 0, sizeof(event));
  event.type = KeyPress;
  event.xkey.same_screen = true;
  event.xkey.type = KeyPress;
  event.xkey.keycode = ::XKeysymToKeycode(this->main_display, symb);

  ::XQueryPointer(this->main_display, RootWindow(this->main_display, DefaultScreen(this->main_display)), \
		  &event.xbutton.root, &event.xbutton.window, &event.xbutton.x_root, \
		  &event.xbutton.y_root, &event.xbutton.x, &event.xbutton.y, &event.xbutton.state);
  event.xkey.subwindow = event.xbutton.window;

  while (event.xkey.subwindow)
    {
      event.xkey.window = event.xkey.subwindow;
      ::XQueryPointer(this->main_display, event.xkey.window, &event.xkey.root, &event.xkey.subwindow,
		      &event.xkey.x_root, &event.xkey.y_root, &event.xkey.x, &event.xkey.y, &event.xkey.state);
    }

  ::XSendEvent(this->main_display, PointerWindow, propagate, KeyPressMask, &event);
  ::XFlush(this->main_display);
  event.xkey.state = 0x100;
}

void XAction::keyRelease(int k)
{
  bool propagate = true;
  KeySym symb = k;
  XEvent event;

  memset(&event, 0, sizeof(event));
  event.type = KeyRelease;
  event.xkey.same_screen = true;
  event.xkey.type = KeyRelease;
  event.xkey.keycode = ::XKeysymToKeycode(this->main_display, symb);

  ::XQueryPointer(this->main_display, RootWindow(this->main_display, DefaultScreen(this->main_display)), \
		  &event.xbutton.root, &event.xbutton.window, &event.xbutton.x_root, \
		  &event.xbutton.y_root, &event.xbutton.x, &event.xbutton.y, &event.xbutton.state);
  event.xkey.subwindow = event.xbutton.window;

  while (event.xkey.subwindow)
    {
      event.xkey.window = event.xkey.subwindow;
      ::XQueryPointer(this->main_display, event.xkey.window, &event.xkey.root, &event.xkey.subwindow,
		      &event.xkey.x_root, &event.xkey.y_root, &event.xkey.x, &event.xkey.y, &event.xkey.state);
    }

  ::XSendEvent(this->main_display ,PointerWindow, propagate, KeyReleaseMask, &event);
  ::XFlush(this->main_display);
  event.xkey.state = 0x100;
}

#endif //!_WIN32
